#include "hmi.h"

#define STATE_START 10
#define STATE_PLAY 20
#define STATE_RECORD 30
#define STATE_FAILURE 40
#define STATE_SUCCESS 50

#define TIMEOUT 2000

int state = STATE_START;
byte sequence[100];
int sequence_index = 0;
int level = 0;

void setup() {
  Serial.begin(9600);
  hmi_setup();
  for(int i = 0; i < 4; ++i) {
    hmi_flash(i, 1.0);
  }
  randomSeed(analogRead(A0) ^ analogRead(A1) ^ analogRead(A2));
  delay(1000);
}

void loop() {
  switch(state) {
    case STATE_START:
      state = handle_start();
      break;
    case STATE_PLAY:
      state = handle_play();
      break;
    case STATE_RECORD:
      state = handle_record();
      break;
    case STATE_SUCCESS:
      state = handle_success();
      break;
    case STATE_FAILURE:
      state = handle_failure();
      break;
  }
}

int handle_start() {
  Serial.println("handle_start()");
  add_note();
  return STATE_PLAY;
}

int handle_play() {
  Serial.println("handle_play()");
  for(int i = 0; i < sequence_index; ++i) {
    hmi_flash(sequence[i], 1.0);
  }
  return STATE_RECORD;
}

int handle_record() {
  Serial.println("handle_record()");
  int record_index = 0;
  boolean success = true;
  int user_press;
  while(success && record_index < sequence_index) {
    Serial.print("expecting user input#"); Serial.print(record_index); Serial.print(": "); Serial.println(sequence[record_index]);
    user_press = hmi_get_user_selection(TIMEOUT);
    success = (user_press == sequence[record_index]);
    if(success) {
      hmi_flash(user_press, 1.0);
      record_index++;
    }
  }
  return success ? STATE_SUCCESS : STATE_FAILURE;
}

int handle_success() {
  Serial.println("handle_success()");
  add_note();
  if(sequence_index % 7 == 0) {
    hmi_on_new_level();
    level++;
  }
  delay(1000);
  return STATE_PLAY;
}

int handle_failure() {
  Serial.println("handle_failure()");
  hmi_on_failure();
  reset();
  delay(1000);
  return STATE_START;
}

void add_note() {
  Serial.println("add_note");
  sequence[sequence_index++] = random(0, 4);
  for(int i = 0; i < sequence_index; ++i) {
    Serial.print(sequence[i]); Serial.print("\t"); 
  }
  Serial.println();
}

void reset() {
  Serial.println("reset");
  level = 0;
  sequence_index = 0;
}

