#include <Arduino.h>
#include "pitches.h"

#define SPEAKER_PIN 12

const int switchPins[] = {
  2, 3, 4, 5};
  
const int ledPins[] = {
  6, 7, 8, 9};
const int NOTES[] = {
  NOTE_C4, NOTE_E4, NOTE_G4, NOTE_C5 };

void hmi_setup() {
  // setup buttons, LEDs, etc…
  for(int i = 0; i < 4; ++i) {
    pinMode(switchPins[i], INPUT_PULLUP);
    pinMode(ledPins[i], OUTPUT);
  }
}

void hmi_flash(int idx, float speedd) {
  digitalWrite(ledPins[idx], HIGH);
  tone(SPEAKER_PIN, NOTES[idx]);
  delay(400 * speedd);
  noTone(SPEAKER_PIN);
  digitalWrite(ledPins[idx], LOW);
  delay(75 * speedd);
}

float majScale[] = {
	A,  B,  CS,  D,  E,  FS,  GS,  A2_,   B2,  C2S,  D2,  E2,  F2S,  G2S,  A3_};

void cheesy_effect() { 
  int ptime;
  int   k,  x, dur, freq, t; 
  int i, j;
  float ps;         // variable for pow pitchShift routine
  float noteval;
  //note values

  for(i= 0; i<=4; i++){ 
    ps = (float)i / 12;         // choose new transpose interval every loop
    digitalWrite(ledPins[i], HIGH);
    for(x= 0; x<=15; x++){                      
      noteval = (majScale[x] / 64) * pow(2,ps);    // transpose scale up 12 tones - pow function generates transpostion
      dur = 50 - i * 8 - x;
      tone(SPEAKER_PIN, (int)noteval, dur);
      delay(dur * 1.1);
    }
    digitalWrite(ledPins[i], LOW);
  }
  for(i= 0; i<=6; i++){ 
    ps = (float)i / 12;         // choose new transpose interval every loop
    digitalWrite(ledPins[i], HIGH);
    for(x= 0; x<=15; x++){                      
      noteval = (majScale[x] / 64) * pow(2,ps);    // transpose scale up 12 tones - pow function generates transpostion
      dur = 5;
      tone(SPEAKER_PIN, (int)noteval, dur);
      delay(dur * 1.1);
    }
    digitalWrite(ledPins[i], LOW);
  }
} 

int hmi_get_user_selection(int timeout) {
  int result = -1;
  long startt = millis();
  while(millis() - startt < timeout && (result == -1)) {
    for(int i = 0; i < 4; ++i) {
      if(digitalRead(switchPins[i]) == LOW) {
        result = i;
        break;
      }
    }
  }  
  Serial.print("user selected: "); Serial.println(result);
  return result;
}

void hmi_on_failure() {
  Serial.println("hmi_on_failure()");
  int i = 0;
//  for(int t = 0; t < 3; ++t) {
    for(int i = 0; i < 4; ++i) {
      int j;
      for(j = 50; j > 0; j--) {
        if(j%5==0) {
          digitalWrite(ledPins[j%4], !digitalRead(ledPins[j%4]));
        }
        tone(SPEAKER_PIN, (5 - i) * 200 + j * 20);
//        delayMicroseconds(500*(4-t));
        delay(7);
      }
    }
    for(int i = 0; i < 4; ++i) {
      digitalWrite(ledPins[i], LOW);
    }
    noTone(SPEAKER_PIN);
//  }
}

void hmi_on_new_level() {
  cheesy_effect();
}

